@@ -3,4 +3,6 @@ from .SPGNN.main import SPGNN
 from .mol2vec.main import Mol2Vec
 from .piCalculax import calc_pI
 from .stoned import *
-from .ChemProp.main import ChemPropModel
\ No newline at end of file
+from .ChemProp.main import ChemPropModel
+from .HondaSTRep.main import HondaSTRep
+from .HondaSTRep.operations import WordVocab
\ No newline at end of file
