@@ -1,4 +1,5 @@
 from cmath import e
+from lib2to3.pgen2.literals import simple_escapes
 import os
 from re import escape
 import urllib.parse
@@ -51,8 +52,8 @@ class BaseVisualization(BaseClass):
     # A list of packages that can be specified in self.packages to be loaded
     package_urls = {
         "d3": "https://d3js.org/d3.v4.js",
-        "plotly": "https://cdn.plot.ly/plotly-2.12.1.min.js",
-        "smilesdrawer": "https://unpkg.com/smiles-drawer@1.0.10/dist/smiles-drawer.min.js",
+        "plotly": "https://cdn.plot.ly/plotly-2.14.0.min.js",
+        "olorenrenderer": "https://unpkg.com/olorenrenderer@1.0.0-c/dist/oloren-renderer.min.js",
         "rdkit": "https://unpkg.com/@rdkit/rdkit/dist/RDKit_minimal.js",
     }
 
@@ -162,7 +163,7 @@ class BaseVisualization(BaseClass):
         # Formats the data to be properly JSON
         if data is None:
             data_str = (
-                json.dumps(self.get_data())
+                json.dumps(self.get_data(), separators=(',', ':'))
                 .replace('"', "&quot;")
                 .replace("None", "null")
                 .replace("NaN", "null")
@@ -171,7 +172,7 @@ class BaseVisualization(BaseClass):
             )
         else:
             data_str = (
-                json.dumps(data)
+                json.dumps(data, separators=(',', ':'))
                 .replace('"', "&quot;")
                 .replace("None", "null")
                 .replace("NaN", "null")
@@ -180,8 +181,7 @@ class BaseVisualization(BaseClass):
             )
 
         # Compiles package imports into HTML
-        packages = "".join([f'<script src="{self.package_urls[package]}"></script>' for package in self.packages])
-
+        packages = "".join([f'<script src = "{self.package_urls[package]}"></script>' for package in self.packages])
         # Get base HTML code for visualization
         html = self.get_html(data_str, packages)
 
@@ -334,7 +334,7 @@ class VisualizeError(BaseVisualization):
 
         return d
 
-class VisualizeDatasetCompounds(BaseVisualization):
+class VisualizeCompounds(BaseVisualization):
     """Visualizes the compounds in a set.
 
     Parameters:
@@ -347,20 +347,29 @@ class VisualizeDatasetCompounds(BaseVisualization):
         compound_height = 250 (int): Height of each compound image in the table
         annotations = None (str): Name of columns in dataset to be used to annotated
             the table. Will only be considered if dataset is a BaseDataset.
+        kekulize = True (bool): Whether or not to kekulize molecules for rendering.
+            Default is True.
         box = True (bool):
         shuffle = True (bool): Whether or not to shuffle the compounds."""
 
     @log_arguments
-    def __init__(self, dataset: Union[BaseDataset, list, pd.Series], table_width: int = 2, table_height: int = 5,
-        compound_width: int = 250, compound_height: int = 250, annotations = None,
-        box=True, shuffle=True,log=True, **kwargs):
+    def __init__(self, dataset: Union[BaseDataset, list, pd.Series, str, Chem.Mol], table_width: int = 1, table_height: int = 5,
+        compound_width: int = 500, compound_height: int = 500, annotations = None, kekulize = True,
+        box=False, shuffle=True,log=True, **kwargs):
         self.dataset = dataset
         if issubclass(type(dataset), BaseDataset):
             self.compounds = dataset.data[dataset.structure_col].head(table_width * table_height).tolist()
         elif isinstance(dataset, pd.Series):
             self.compounds = dataset.tolist()
         elif isinstance(dataset, list):
-            self.compounds = dataset
+            if isinstance(dataset[0], str):
+                self.compounds = dataset
+            if isinstance(dataset[0], Chem.Mol):
+                self.compounds = [Chem.MolToSmiles(m) for m in dataset]
+        elif isinstance(dataset, str):
+            self.compounds = [dataset]
+        elif issubclass(type(dataset), Chem.Mol):
+            self.compounds = [Chem.MolToSmiles(dataset)]
         if shuffle:
             import random
             random.shuffle(self.compounds)
@@ -377,11 +386,14 @@ class VisualizeDatasetCompounds(BaseVisualization):
         self.compound_width = compound_width
         self.compound_height = compound_height
         self.annotations = annotations
+        self.kekulize = kekulize
         self.box = box
         super().__init__(**kwargs)
-        self.packages = ["smilesdrawer", "plotly"]
+        self.packages = ["olorenrenderer", "plotly"]
 
     def get_data(self):
+        if self.kekulize:
+            self.compounds = [Chem.MolToSmiles(Chem.MolFromSmiles(s), kekuleSmiles = True) for s in self.compounds]
         d =  {"smiles": self.compounds,
             "table_width": self.table_width,
             "table_height": self.table_height,
@@ -394,6 +406,9 @@ class VisualizeDatasetCompounds(BaseVisualization):
 
         return d
 
+class VisualizeDatasetCompounds(VisualizeCompounds):
+    """Alias for VisualizeCompounds"""
+    pass
 
 class ScatterPlot(BaseVisualization):
     """Scatter plot visualization.
@@ -432,6 +447,8 @@ class CompoundScatterPlot(BaseVisualization):
             y-axis value. Default is None.
         smiles_col (str, optional): If specified, uses value as the column name for the
             molecule smiles. Default is None.
+        kekulize (bool, optional): Whether or not to kekulize molecules for display.
+            Default is True.
         color_col (str, optional): If specified, uses value as the column name for the
             color of the markers. Default is None.
         xaxis_type (str, optional): Type of x-axis. Default is 'linear', other
@@ -470,7 +487,9 @@ class CompoundScatterPlot(BaseVisualization):
         x_col: str =    None,
         y_col: str =    None,
         smiles_col: str = None,
+        kekulize: bool = True,
         color_col: str = None,
+        colorscale: str = None,
         xaxis_type: str = "linear",
         yaxis_type: str = "linear",
         axesratio: float = None,
@@ -503,10 +522,16 @@ class CompoundScatterPlot(BaseVisualization):
             self.df["SMILES"] = self.df[smiles_col]
             if smiles_col != "SMILES":
                 self.df = self.df.drop(smiles_col, axis=1)
+
+        if color_col == "property_col":
+            self.color_col = self.dataset.property_col
+        else:
+            self.color_col = color_col
         if color_col is not None:
             self.df["color"] = self.df[color_col]
             if color_col != "color":
                 self.df = self.df.drop(color_col, axis=1)
+        self.colorscale = colorscale
 
         # Sets up axes titling using column names as defaults if available
         if xaxis_title is None and x_col is not None:
@@ -519,6 +544,7 @@ class CompoundScatterPlot(BaseVisualization):
             self.yaxis_title = yaxis_title
 
         # Saves aesthetic variables
+        self.kekulize = kekulize
         self.xaxis_type = xaxis_type
         self.yaxis_type = yaxis_type
         self.axesratio = axesratio
@@ -543,7 +569,7 @@ class CompoundScatterPlot(BaseVisualization):
         super().__init__(log=False, **kwargs)
 
         # Add packages to import for JavaScript
-        self.packages += ["plotly", "smilesdrawer"]
+        self.packages += ["plotly", "olorenrenderer"]
 
     @property
     def JS_NAME(self) -> str:
@@ -560,6 +586,7 @@ class CompoundScatterPlot(BaseVisualization):
 
         if include_data:
             d = self.df.to_dict("l")
+            d["SMILES"] = self.df["SMILES"].apply(lambda x: Chem.MolToSmiles(Chem.MolFromSmiles(x), kekuleSmiles=self.kekulize)).tolist()
         else:
             d = dict()
         d["title"] = self.title
@@ -571,6 +598,8 @@ class CompoundScatterPlot(BaseVisualization):
         d["width"] = self.width
         d["height"] = self.height
         d["opacity"] = self.opacity
+        if not self.colorscale is None:
+            d["colorscale"] = self.colorscale
 
         if self.axesratio is not None:
             d["axesratio"] = self.axesratio
@@ -609,7 +638,9 @@ class ChemicalSpacePlot(CompoundScatterPlot):
     performing dimensionality reduction to 2 dimensions.
 
     Parameters:
-        dataset (BaseDataset): Dataset to be used in visualization.
+        dataset (BaseDataset, pd.Seriess, list): BaseDataset to be used in visualization. Alternatively
+            can be a list or pd.Series where then this object will be treated as a list
+            of structures.
         rep (BaseCompoundVecRepresentation): Representation to use for dimensionality reduction.
         dim_reduction (str, optional): Dimensionality reduction method to use. Default is
             'tsne' other options are 'pca'.
@@ -622,21 +653,23 @@ class ChemicalSpacePlot(CompoundScatterPlot):
         """
 
     @log_arguments
-    def __init__(self, dataset: BaseDataset, rep: BaseCompoundVecRepresentation, dim_reduction="tsne", *args,
-            color = None, colorscale = "Portland", title ="Chemical Space Plot", log=True, **kwargs):
+    def __init__(self, dataset: Union[BaseDataset, list, pd.Series, pd.DataFrame], rep: BaseCompoundVecRepresentation,
+            *args, dim_reduction="tsne",
+            smiles_col=None, title ="Chemical Space Plot", log=True, **kwargs):
 
         # Sets visualization instance variables
-        self.dataset = dataset
-        self.colorscale = colorscale
+        if issubclass(type(dataset), BaseDataset):
+            self.structures = dataset.data[dataset.structure_col]
+        elif isinstance(dataset, pd.DataFrame):
+            assert smiles_col is not None, "smiles_col must be defined if `dataset` parameter is pd.DataFrame"
+            self.structures = dataset[smiles_col]
+        else:
+            self.structures = dataset
         self.rep = rep
         self.dim_reduction = dim_reduction
-        if color == "property_col":
-            self.color = self.dataset.property_col
-        else:
-            self.color = color
 
         # Converts the molecules in the dataset to the desired representation
-        chem_rep_list = self.rep.convert(self.dataset.data[self.dataset.structure_col])
+        chem_rep_list = self.rep.convert(self.structures)
 
         # Does dimensionality reduction on the given representation to get the
         # 2D coordinates of the chemical space plot
@@ -646,14 +679,22 @@ class ChemicalSpacePlot(CompoundScatterPlot):
             df = self.pca_df(chem_rep_list)
 
         # Sets the dataframe up to be used by the parent class CompoundScatterPlot
-        self.dataset.data["X"] = df["Component 1"]
-        self.dataset.data["Y"] = df["Component 2"]
-        self.dataset.data["SMILES"] = self.dataset.data[self.dataset.structure_col]
+        if issubclass(type(dataset), BaseDataset):
+            self.df = dataset.data
+        elif issubclass(type(dataset), pd.DataFrame):
+            self.df = dataset
+        else:
+            self.df = pd.DataFrame()
+
+        self.df["X"] = df["Component 1"]
+        self.df["Y"] = df["Component 2"]
+        self.df["SMILES"] = self.structures
 
         self.title = title
 
-        super().__init__(self.dataset.data, *args, title=self.title,
-            xaxis_title="Component 1", yaxis_title="Component 2", log=False, **kwargs)
+        super().__init__(self.df, *args, title=self.title,
+            xaxis_title="Component 1", yaxis_title="Component 2", smiles_col = smiles_col,
+            log=False, **kwargs)
 
     def tsne_df(self, chem_rep_list):
         """
@@ -685,25 +726,75 @@ class ChemicalSpacePlot(CompoundScatterPlot):
         pca_arr = pca.fit_transform(chem_rep_list)
         return pd.DataFrame(pca_arr, columns=["Component 1", "Component 2"])
 
-    def get_data(self, color: str = None, size: str = None, SMILES: str = None) -> dict:
-        d = super().get_data(include_data=False)
+    def get_data(self, color_col: str = None, size_col: str = None, SMILES: str = None) -> dict:
+        d = super().get_data(include_data=True)
 
-        if not self.color is None:
-            assert self.color in self.dataset.data.columns, f"specified color column, {color}, not in columns"
-            d["color"] = self.dataset.data[self.color].tolist()
+        self.color_col = color_col
+        if not self.color_col is None:
+            assert self.color_col in self.df.columns, f"specified color column, {self.color_col}, not in columns"
+            d["color"] = self.df[self.color_col].tolist()
 
-        if not size is None:
-            assert size in self.df.columns, f"specified size column, {size}, not in columns"
-            d["size"] = self.df[size].tolist()
-
-        d["X"] = self.dataset.data["X"].tolist()
-        d["Y"] = self.dataset.data["Y"].tolist()
-        d["SMILES"] = self.dataset.data[self.dataset.structure_col].tolist()
-
-        d["colorscale"] = self.colorscale
+        self.size_col = size_col
+        if not self.size_col is None:
+            assert self.size_col in self.df.columns, f"specified size column, {self.size_col}, not in columns"
+            d["size"] = self.df[self.size_col].tolist()
 
         return d
 
+class VisualizeMoleculePerturbations(ChemicalSpacePlot):
+    """ Visualize perturbations of a single molecule given from a PerturbationEngine
+    in a ChemicalSpacePlot.
+
+    Parameters:
+        smiles (str): SMILES of molecule to perturb.
+        perturbation_engine (PerturbationEngine): Perturbation engine, which has
+            the underlying algorithm for perturbing molecules. Default is `SwapMutations(radius = 0)`
+        rep (BaseVecRepresentation): Molecular vector representation to use for
+            dimensionality reduction"""
+
+    @log_arguments
+    def __init__(self, smiles: str,
+            perturbation_engine: PerturbationEngine = None,
+            rep: BaseVecRepresentation = None,
+            idx: int = None,
+            n: int = None):
+        self.smiles = smiles
+        if perturbation_engine is None:
+            self.perturbation_engine = SwapMutations(radius = 0)
+        else:
+            self.perturbation_engine = perturbation_engine
+        if rep is None:
+            self.rep = DescriptastorusDescriptor("morgan3counts")
+        else:
+            self.rep = rep
+
+        if n is None:
+            self.n = 100
+        else:
+            self.n = n
+
+        from rdkit import Chem
+        from rdkit.Chem import AllChem
+
+        df = pd.DataFrame()
+        if idx is None:
+            df["SMILES"] = self.perturbation_engine.get_compound_list(smiles) + [smiles]
+        else:
+            df["SMILES"] = self.perturbation_engine.get_compound_list(smiles, idx = idx)+ [smiles]
+        df["mols"] = [Chem.MolFromSmiles(s) for s in df["SMILES"]]
+        df = df.dropna(subset = ["mols"])
+
+        fps = [AllChem.GetMorganFingerprintAsBitVect(m, 2, nBits=2048, useChirality=False) for m in df["mols"]]
+
+        df["sim"] = DataStructs.BulkTanimotoSimilarity(AllChem.GetMorganFingerprintAsBitVect(Chem.MolFromSmiles(self.smiles), 2, nBits=2048, useChirality=False) , fps)
+
+        super().__init__(df,
+                self.rep,
+                title = "Chemical Space Plot of Molecular Perturbations<br><sub>Points colored by tanimoto similarity to the reference compund</sub>",
+                color_col = "sim",
+                smiles_col = "SMILES",
+                colorscale = "YlOrRd")
+
 class VisualizeDatasetSplit(ChemicalSpacePlot):
     """Visualize a dataset by seeing where train/test compounds are in a dimensionality
     reduced space by coloring the compounds by whether or not they are in
@@ -852,7 +943,7 @@ class MorganContributions(BaseVisualization):
         self.original_prediction, self.predictions = self._make_predictions(self.smiles)
         self.args = args
         self.kwargs = kwargs
-        self.packages = ["plotly", "rdkit", "smilesdrawer"]
+        self.packages = ["plotly", "rdkit", "olorenrenderer"]
 
     def _train_model(self):
         """ Train random forest model based on the morgan vec representation"""
@@ -1284,12 +1375,14 @@ class VisualizeCounterfactual(CompoundScatterPlot):
         self,
         smiles: str,
         model: BaseModel,
-        perturbation_engine: PerturbationEngine = SwapMutations(radius=1),
+        perturbation_engine: PerturbationEngine = None,
         delta: Union[int, float, Tuple] = (-1, 1),
         n: int = 40,
         pca: bool = False,
         **kwargs,
     ):
+        if perturbation_engine is None:
+            perturbation_engine = SwapMutations(radius=1)
 
         self.smiles = smiles
         self.model = model
@@ -1528,6 +1621,74 @@ class ModelROC(BaseVisualization):
         d["score"] = np.around(self.score, decimals=2)
         return d
 
+class ModelROCThreshold(BaseVisualization):
+    """ Visualize the ROC curve for a model.
+
+    Parameters:
+        dataset (Dataset): Dataset to evaluate model on
+        model (BaseModel): Model to evaluate on
+        model_name (str, optional): name of model. Default is None.
+        eval_set (str, optional): Subset of dataset to visualize, either 'train', 'test',
+            or 'valid'. Default is 'test'."""
+
+    @log_arguments
+    def __init__(self, dataset: BaseDataset, model: BaseModel, *args, eval_set="test",
+        log=True, model_name=None, **kwargs):
+
+        if not type(dataset) is BaseDataset:
+            raise TypeError(
+                "dataset must be a BaseDataset. If you're using a Pandas Dataframe consider using df.to_csv() in the"
+                + " oce.BaseDataset class to convert a pandas.DataFrame of data to a BaseDataset."
+            )
+        self.dataset = dataset
+        self.model = model
+
+        if eval_set == "test":
+            self.eval_set = self.dataset.test_dataset
+        elif eval_set == "train":
+            self.eval_set = self.dataset.train_dataset
+        elif eval_set == "valid":
+            self.eval_set = self.dataset.valid_dataset
+        else:
+            self.eval_set = self.dataset.entire_dataset
+
+        preds = self.model.predict(self.eval_set[0])
+
+        from sklearn import metrics
+
+        y_true = self.eval_set[1]
+        if isinstance(y_true, pd.Series):
+            y_true = y_true.tolist()
+
+        fpr, tpr, thresholds = metrics.roc_curve(y_true, preds)
+        ix = np.sort(np.random.choice(np.arange(len(fpr)), size=1000, replace=True))
+        self.df = pd.DataFrame({"X": fpr[ix], "Y": tpr[ix],
+            "thresholds": thresholds[ix]},)
+
+        self.score = metrics.roc_auc_score(y_true, preds)
+
+        if model_name is None:
+            self.model_name = model_name_from_params(oce.parameterize(model))
+        else:
+            self.model_name = model_name
+
+        self.packages = ["plotly"]
+
+    @staticmethod
+    def get_attributes():
+        return [{"Select a Model/Dataset": [["model", "modelSelector"], ["dataset", "datasetSelector"],]}]
+
+    @property
+    def JS_NAME(self) -> str:
+            return "ModelROCThreshold"
+
+    def get_data(self) -> dict:
+        d = self.df.to_dict("l")
+        d["model_name"] = self.model_name
+        d["score"] = np.around(self.score, decimals=2)
+        d["P"] = (sum(self.dataset.test_dataset[1]))/len(self.dataset.test_dataset[1])
+        d["N"] = (len(self.dataset.test_dataset[1]) - sum(self.dataset.test_dataset[1]))/len(self.dataset.test_dataset[1])
+        return d
 
 class ModelPR(BaseVisualization):
     """ Visualize the Precision-Recall curve for a model.
@@ -1578,4 +1739,4 @@ class ModelPR(BaseVisualization):
         d["model_name"] = self.model_name
         d["score"] = np.around(self.score, decimals=2)
         d["baseline"] = self.baseline
-        return d
+        return d
\ No newline at end of file
